import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

public class FormMapWithSetWarships extends JFrame{
    private JPanel mainPanel;
    private JPanel PictureBox;
    private JPanel GroupBoxTools;
    private JComboBox СomboBoxSelectorMap;
    private JButton ButtonAddWarship;
    private JButton ButtonRemoveWarship;
    private JButton ButtonShowStorage;
    private JButton ButtonShowOnMap;
    private JTextField TextBoxPosition;
    private JButton ButtonRight;
    private JButton ButtonUp;
    private JButton ButtonDown;
    private JButton ButtonLeft;
    private JButton deletedWarshipButtom;
    private JPanel MapPanel;
    private JTextField TextFieldMap;
    private JButton CreateMapButton;
    private JButton DeleteMapButton;
    private JList ListBoxMaps;
    private Image bufferedImage;
    private MapWithSetWarshipsGeneric<DrawingObjectWarship, AbstractMap> _mapWarshipsCollectionGeneric;
    private MapsCollection _mapsCollection;
    private final HashMap<String, AbstractMap> _mapsDict = new HashMap<>() {{
        put("Простая карта", new SimpleMap());
        put("Преграды-линии", new LineMap());
    }};

    public FormMapWithSetWarships(){
        InitializeComponent();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        if (bufferedImage != null) {
            PictureBox.paintComponents(bufferedImage.getGraphics());
            PictureBox.getGraphics().drawImage(bufferedImage, 0, 0, null);
        }
    }

    private void ReloadMaps()
    {
        int index = ListBoxMaps.getSelectedIndex();
        ListBoxMaps.setListData(_mapsCollection.Keys().toArray(new String[0]));

        if (ListBoxMaps.getModel().getSize() > 0 && (index == -1 || index >= ListBoxMaps.getModel().getSize()))
        {
            ListBoxMaps.setSelectedIndex(0);
        }
        else if (ListBoxMaps.getModel().getSize() > 0 && index > -1 && index < ListBoxMaps.getModel().getSize())
        {
            ListBoxMaps.setSelectedIndex(index);
        }
        repaint();
    }

    private void InitializeComponent(){
        setContentPane(mainPanel);
        setTitle("Warship");
        setSize(935, 693);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setVisible(true);

        _mapsCollection = new MapsCollection(getWidth(), getHeight());
        СomboBoxSelectorMap.removeAllItems();
        for(String elem:_mapsDict.keySet()){
            СomboBoxSelectorMap.addItem(elem);
        }

        Icon iconUp = new ImageIcon("src\\Images\\ArrowUp.jpg");
        ButtonUp.setIcon(iconUp);
        Icon iconDown = new ImageIcon("src\\Images\\ArrowDown.jpg");
        ButtonDown.setIcon(iconDown);
        Icon iconLeft = new ImageIcon("src\\Images\\ArrowLeft.jpg");
        ButtonLeft.setIcon(iconLeft);
        Icon iconRight = new ImageIcon("src\\Images\\ArrowRight.jpg");
        ButtonRight.setIcon(iconRight);

        ListBoxMaps.addListSelectionListener(e -> {
            if(ListBoxMaps.getSelectedIndex() == -1)
                return;
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
        });

        DeleteMapButton.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() == -1)
            {
                return;
            }
            if(JOptionPane.showConfirmDialog(this,"Удалить карту " + ListBoxMaps.getSelectedValue().toString() + "?",
                    "Удаление",JOptionPane.YES_NO_OPTION) == 0)
            {
                _mapsCollection.DelMap(ListBoxMaps.getSelectedValue().toString());
                ReloadMaps();
            }
        });

        CreateMapButton.addActionListener(e -> {
            if (СomboBoxSelectorMap.getSelectedIndex() == -1 || TextFieldMap.getText() == null || TextFieldMap.getText().equals(""))
            {
                JOptionPane.showMessageDialog(this,"Не все данные заполнены","Ошибка",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (!_mapsDict.containsKey(СomboBoxSelectorMap.getSelectedItem()))
            {
                JOptionPane.showMessageDialog(this,"Нет такой карты","Ошибка",JOptionPane.ERROR_MESSAGE);
                return;
            }
            _mapsCollection.AddMap(TextFieldMap.getText(), _mapsDict.get(СomboBoxSelectorMap.getSelectedItem().toString()));
            ReloadMaps();
        });

        deletedWarshipButtom.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() == -1)
                return;
            DrawingObjectWarship warship = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).GetWarshipInDeleted();
            if (warship == null) {
                JOptionPane.showMessageDialog(null, "Стек пуст", "Ошибка", JOptionPane.ERROR_MESSAGE);
                return;
            }
            FormWarship form = new FormWarship(warship);
            form.setSize(1000, 700);
            form.setVisible(true);
        });

        ButtonAddWarship.addActionListener(e -> {
            if(ListBoxMaps.getSelectedIndex() == -1){
                return;
            }
            FormWarshipCreator warshipCreator = new FormWarshipCreator();
            warshipCreator.setSize(1200,700);
            warshipCreator.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
            warshipCreator.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
            warshipCreator.setVisible(true);
            if (warshipCreator.getSelectedWarship() != null) {
                DrawingObjectWarship warship = new DrawingObjectWarship(warshipCreator.getSelectedWarship());

                if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).Plus(warship)>=0) {
                    JOptionPane.showMessageDialog(this,
                            "Объект добавлен", "Успех", JOptionPane.INFORMATION_MESSAGE);
                    bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                    repaint();
                } else {
                    JOptionPane.showMessageDialog(this,
                            "Не удалось добавить объект", "Ошибка",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        });

        ButtonRemoveWarship.addActionListener(e -> {

            if (TextBoxPosition.getText().isEmpty())
            {
                return;
            }
            int result = JOptionPane.showConfirmDialog(null,
                    "Удалить объект?",
                    "Удаление",
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.WARNING_MESSAGE);
            if (result == JOptionPane.NO_OPTION)
            {
                return;
            }
            int pos = Integer.parseInt(TextBoxPosition.getText());
            if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).Minus(pos)!=null)
            {
                JOptionPane.showMessageDialog(this,
                        "Объект удален","Успех",JOptionPane.INFORMATION_MESSAGE);
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                repaint();
            }
            else
            {
                JOptionPane.showMessageDialog(this,
                        "Не удалось удалить объект","Ошибка",JOptionPane.INFORMATION_MESSAGE);
            }
        });

        ButtonShowStorage.addActionListener(e -> {
            if(_mapWarshipsCollectionGeneric == null)
            {
                return;
            }
            bufferedImage = _mapWarshipsCollectionGeneric.ShowSet();
            repaint();
        });

        ButtonShowOnMap.addActionListener(e -> {
            if (_mapWarshipsCollectionGeneric == null)
            {
                return;
            }
            bufferedImage = _mapWarshipsCollectionGeneric.ShowOnMap();
            repaint();
        });

        ButtonUp.addActionListener(e -> {
            if (_mapWarshipsCollectionGeneric != null) {
                bufferedImage = _mapWarshipsCollectionGeneric.MoveObject(Direction.Up);
                repaint();
            }
        });

        ButtonDown.addActionListener(e -> {
            if (_mapWarshipsCollectionGeneric != null) {
                bufferedImage = _mapWarshipsCollectionGeneric.MoveObject(Direction.Down);
                repaint();
            }
        });

        ButtonRight.addActionListener(e -> {
            if (_mapWarshipsCollectionGeneric != null) {
                bufferedImage = _mapWarshipsCollectionGeneric.MoveObject(Direction.Right);
                repaint();
            }
        });

        ButtonLeft.addActionListener(e -> {
            if (_mapWarshipsCollectionGeneric != null) {
                bufferedImage = _mapWarshipsCollectionGeneric.MoveObject(Direction.Left);
                repaint();
            }
        });
    }
}
