import javax.swing.*;
import java.awt.*;
import java.util.Random;

public class FormWarshipCreator extends JDialog{
    private JPanel PictureBox;
    private JPanel SettingsPanel;
    private JPanel BlocksTypePanel;
    private JPanel CountOfBlocksPanel;
    private JPanel ModificationPanel;
    private JPanel TypePanel;
    private JRadioButton RoundRectangleFormRadioButton;
    private JRadioButton RoundFormRadioButton;
    private JRadioButton RectangleFormRadioButton;
    private JRadioButton SixRadioButton;
    private JRadioButton FourRadioButton;
    private JRadioButton TwoRadioButton;
    private JButton CreateButton;
    private JButton SelectButton;
    private JCheckBox EngineCheckBox;
    private JCheckBox CabinCheckBox;
    private JCheckBox AreaCheckBox;
    private JComboBox comboBoxType;
    private BlockCount _block = null;
    private IDrawingObjectBlock objectBlock = null;
    private final WarshipCreatorGeneric<EntityWarship,IDrawingObjectBlock> warshipCreator =new WarshipCreatorGeneric<>(40,40);
    private DrawingWarship _warship;
    private DrawingWarship selectedWarship;
    private boolean isModified = false;

    public FormWarshipCreator(){
        InitializeComponent();
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) PictureBox.getGraphics();
        if (_warship != null) {
            _warship.DrawTransport(g2d);
        }
    }

    public DrawingWarship getSelectedWarship() {
        return selectedWarship;
    }

    private void InitializeComponent(){
        setContentPane(PictureBox);
        setTitle("Warship");
        setResizable(false);
        setSize(1000, 600);

        //Определение типа объекта
        comboBoxType.addItemListener(e -> isModified = e.getItem().toString().equals("AircraftCarrier"));

        //количество блоков
        TwoRadioButton.addActionListener(e -> {
            _block = BlockCount.TwoBlocks;
        });
        FourRadioButton.addActionListener(e -> {
            _block = BlockCount.FourBlocks;
        });
        SixRadioButton.addActionListener(e -> {
            _block = BlockCount.SixBlocks;
        });

        //Квадратная форма блоков
        RectangleFormRadioButton.addActionListener(e -> {
            if(_block == null) return;

            objectBlock = new DrawingBlocks(_block);
            objectBlock.SetBlockCount(_block.GetBlockCount());
            warshipCreator.AddBlock(objectBlock);
        });

        //Круглая форма блоков
        RoundFormRadioButton.addActionListener(e -> {
            if(_block ==null){
                return;
            }
            objectBlock = new DrawingRoundBlocks(_block);
            objectBlock.SetBlockCount(_block.GetBlockCount());
            warshipCreator.AddBlock(objectBlock);
        });

        //Закруглено-квадратная форма блоков
        RoundRectangleFormRadioButton.addActionListener(e -> {
            if(_block == null){
                return;
            }
            objectBlock = new DrawingRoundBlocks(_block);
            objectBlock.SetBlockCount(_block.GetBlockCount());
            warshipCreator.AddBlock(objectBlock);
        });

        CreateButton.addActionListener(e -> {
            Random rnd = new Random();
            if (_block != null && objectBlock != null) {
                Color color = JColorChooser.showDialog(this, "Choose color", Color.WHITE);

                if (isModified) {
                    Color additionalColor = JColorChooser.showDialog(this, "Choose color", Color.GRAY);

                    warshipCreator.AddWarship(
                            new EntityAircraftCarrier(
                                    rnd.nextInt(10, 300),
                                    rnd.nextFloat(10, 3000),
                                    color,
                                    additionalColor,
                                    AreaCheckBox.isSelected(),
                                    CabinCheckBox.isSelected(),
                                    EngineCheckBox.isSelected()
                            )
                    );
                }
                else {
                    warshipCreator.AddWarship(
                            new EntityWarship(
                                    rnd.nextInt(10, 300),
                                    rnd.nextFloat(10, 3000),
                                    color)
                    );
                }

                _warship = warshipCreator.NewWarshipCreating();
                _warship.SetPosition(
                        rnd.nextInt(20),
                        rnd.nextInt(20),
                        PictureBox.getWidth(),
                        PictureBox.getHeight());
                repaint();
            }
        });
        SelectButton.addActionListener(e -> {
            selectedWarship = _warship;
            dispose();
        });
    }
}
